@@ -1,4 +1,4 @@
-cv.grpreg <- function(X, y, group=1:ncol(X), ..., nfolds=10, seed, trace=FALSE) {
+cv.grpreg <- function(X, y, group=1:ncol(X), ..., nfolds=10, seed, cv.ind, returnY=FALSE, trace=FALSE) {
   if (!missing(seed)) set.seed(seed)
   fit <- grpreg(X=X, y=y, group=group, ...)
   multi <- FALSE
@@ -10,25 +10,27 @@ cv.grpreg <- function(X, y, group=1:ncol(X), ..., nfolds=10, seed, trace=FALSE)
     X <- multiX(X, m)
   }
   g <- if (multi) c(rep(0,m-1), fit$group) else fit$group
-  E <- matrix(NA, nrow=length(y), ncol=length(fit$lambda))
+  E <- Y <- matrix(NA, nrow=length(y), ncol=length(fit$lambda))
   if (fit$family=="binomial") PE <- E
 
   n <- length(y)
-  if (multi) {
-    nn <- n/m
-    cv.ind <- rep(ceiling(sample(1:nn)/nn*nfolds), each=m)
-  } else if (fit$family=="binomial" & (min(table(y)) > nfolds)) {
-    ind1 <- which(y==1)
-    ind0 <- which(y==0)
-    n1 <- length(ind1)
-    n0 <- length(ind0)
-    cv.ind1 <- ceiling(sample(1:n1)/n1*nfolds)
-    cv.ind0 <- ceiling(sample(1:n0)/n0*nfolds)
-    cv.ind <- numeric(n)
-    cv.ind[y==1] <- cv.ind1
-    cv.ind[y==0] <- cv.ind0
-  } else {
-    cv.ind <- ceiling(sample(1:n)/n*nfolds)
+  if (missing(cv.ind)) {
+    if (multi) {
+      nn <- n/m
+      cv.ind <- rep(ceiling(sample(1:nn)/nn*nfolds), each=m)
+    } else if (fit$family=="binomial" & (min(table(y)) > nfolds)) {
+      ind1 <- which(y==1)
+      ind0 <- which(y==0)
+      n1 <- length(ind1)
+      n0 <- length(ind0)
+      cv.ind1 <- ceiling(sample(1:n1)/n1*nfolds)
+      cv.ind0 <- ceiling(sample(1:n0)/n0*nfolds)
+      cv.ind <- numeric(n)
+      cv.ind[y==1] <- cv.ind1
+      cv.ind[y==0] <- cv.ind0
+    } else {
+      cv.ind <- ceiling(sample(1:n)/n*nfolds)
+    }
   }
 
   for (i in 1:nfolds) {
@@ -46,11 +48,13 @@ cv.grpreg <- function(X, y, group=1:ncol(X), ..., nfolds=10, seed, trace=FALSE)
     yhat <- matrix(predict(fit.i, X2, type="response"), length(y2))
     E[cv.ind==i, 1:length(fit.i$lambda)] <- loss.grpreg(y2, yhat, fit$family)
     if (fit$family=="binomial") PE[cv.ind==i, 1:length(fit.i$lambda)] <- (yhat < 0.5) == y2
+    Y[cv.ind==i, 1:length(fit.i$lambda)] <- yhat
   }
 
   ## Eliminate saturated lambda values, if any
   ind <- which(apply(is.finite(E), 2, all))
   E <- E[, ind, drop=FALSE]
+  Y <- Y[,ind]
   lambda <- fit$lambda[ind]
 
   ## Return
@@ -61,5 +65,6 @@ cv.grpreg <- function(X, y, group=1:ncol(X), ..., nfolds=10, seed, trace=FALSE)
 
   val <- list(cve=cve, cvse=cvse, lambda=lambda, fit=fit, min=min, lambda.min=lambda[min], null.dev=null.dev)
   if (fit$family=="binomial") val$pe <- apply(PE[,ind], 2, mean)
+  if (returnY) val$Y <- Y
   structure(val, class="cv.grpreg")
 }
